%******************************************************************************;
%* SAS macro PreZeros vs. 0.2.1 , 28 Oct 2009, by Jim Groeneveld, Netherlands *;
%******************************************************************************;
%*                                                                            *;
   %MACRO   PreZeros   /* Force leading zeros in integer macro variable value */
/* -------------------------------------------------------------------------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Original=           /* Original integer value stored in macro variable   R */
                       /* missing values ._, ., .A - .Z are also supported    */
,  FixedLen= 0         /* Desired total length of resulting macro value     R */
,  CalledBy=PreZeros   /* Macro name of macro or program for err0r report   O */
,  Version = 0.2.1     /* forced version specification for version control  O */
,  ForMyUse=           /* argument for the author's use only, do not use it ! */
)/ DES     =             'Force leading zeros in macro value'     /* STORE */  ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : PREZEROS.SAS                                                  *;
%*                                                                            *;
%* Purpose    : Force leading zeros in integer macro variable values,         *;
%*              similar to %SYSFUNC(PUT(value,Z-format)), but with specific   *;
%*              extra features, converting missing values ._, ., .A-.Z into   *;
%*              __, _0, _A - _Z, prefixed with as many as necessary _ chars,  *;
%*              or _, _, A - Z if FixedLen=1 and not both ._ and . values.    *;
%*              Negative, non-missing values are also prefixed with _ char.   *;
%*              This way legal strings result for postfixing with SAS names.  *;
%*              With FixedLen=0 resulting lengths are variable because no     *;
%*              additional, leading zeros are prefixed, only the significant  *;
%*              digits and additional underscores for negative and missing    *;
%*              values as outlined above (missings length 1, ._ or _ are _).  *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 28 Oct 2009                                                   *;
%* Version    : 0.2.1                                                         *;
%*                                                                            *;
%* SAS version: 6.12 - 8.x and W95 - W2K / 9.1.3 and WXP                      *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Examples                                                                   *;
%* --------                                                                   *;
%*                                                                            *;
/* %PUT <%PreZeros(Original= 3,FixedLen=3)>;                                  */
/* %PUT <%PreZeros(Original= 3,FixedLen=0)>;                                  */
/* %PUT <%PreZeros(Original=-3,FixedLen=3)>;                                  */
/* %PUT <%PreZeros(Original=-3,FixedLen=0)>;                                  */
/* %PUT <%PreZeros(Original=-2,FixedLen=2)>;                                  */
/* %PUT <%PreZeros(Original=. ,FixedLen=0)>;                                  */
/* %PUT <%PreZeros(Original=. ,FixedLen=1)>;                                  */
/* %PUT <%PreZeros(Original=. ,FixedLen=2)>;                                  */
/* %PUT <%PreZeros(Original=. ,FixedLen=3)>;                                  */
/* %PUT <%PreZeros(Original=.n,FixedLen=0)>;                                  */
/* %PUT <%PreZeros(Original=.M,FixedLen=1)>;                                  */
/* %PUT <%PreZeros(Original=.A,FixedLen=2)>;                                  */
/* %PUT <%PreZeros(Original=.z,FixedLen=3)>;                                  */
/* %PUT <%PreZeros(Original=._,FixedLen=3)>;                                  */
/* %PUT <%PreZeros(Original=._,FixedLen=2)>;                                  */
/* %PUT <%PreZeros(Original=._,FixedLen=1)>;                                  */
/* %PUT <%PreZeros(Original=._,FixedLen=0)>;                                  */
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: - [none]                                                       *;
/* = attached: - %MAX                                                         */
%* = external: - [none]                                                       *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of PreZeros                                                        *;
%* ===================                                                        *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0      22 Jun 2001  initial working release                              *;
%* 0.1      19 Nov 2002  mainly cosmetic changes, added parameter checks,     *;
%*                       added CalledBy argument, added version control       *;
%* 0.1.1    10 May 2006  added generating _0 for Original=. if FixedLen>1,    *;
%*                       added generating _ or letter for Original=._ or      *;
%*                          .letter if fixedLen=1, thus if FixedLen=1 then    *;
%*                          no discrimination between . and ._, generating _  *;
%* 0.2.0     8 May 2007  changed SAS code comment into macro code comment     *;
%* 0.2.1    28 Oct 2009  added FixedLen=0 option, specifying a non-fixed      *;
/*                          length, but the value's own widths without zero   */
%*                          prefixes, while keeping the _ prefixes for        *;
%*                          negative and missing values (missing width = 1),  *;
%*                          changed default FixedLen to 0 from empty          *;
%*                       added necessary MAX like macro function as macro     *;
%*                                                                            *;
%******************************************************************************;

%* OPTIONS NOMPRINT NOMERROR NOSERROR NOMLOGIC NOSYMBOLGEN NOMACROGEN;
%* OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
%* remove comment chars for testing ;

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN
  %DO;
    %GOTO Finish;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations ;
%*____________________________________;

 %LOCAL    /* Define internal macro variables explicitely local */
  MacName  /* Name of macro: RemBugNL */
  MacroVs  /* Version number major.minor.patch of macro: 0.2.1 */
  MacDate  /* Date of this version of macro: 28 Oct 2009 */
  OrigLen  /* Length of Original */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;

  %LET MacName = PreZeros;
  %LET MacroVs = 0.2.1;
  %LET MacDate = %STR(28 Oct 2009);
  %LET Err     = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn    = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

%* Macro header in LOG file, dependent on value of CalledBy;
 %IF ( %UPCASE(&CalledBy) EQ %UPCASE(&MacName) ) %THEN
 %DO;
  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT | Macro &MacName, vs. &MacroVs, by Jim Groeneveld, &MacDate |;
  %PUT |    Force leading zeros in integer macro variable value    |;
  %PUT |___________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Original = &Original;
  %PUT &MacName: FixedLen = &FixedLen;
  %PUT &MacName: CalledBy = &CalledBy;
  %PUT &MacName: Version  = &Version ;
  %*PUT &MacName: ForMyUse = &ForMyUse;
 %END;

  %IF (&Version NE AND &Version NE &MacroVs) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Specified version &Version does not match &MacroVs,;
    %PUT .                       macro &MacName will abort;
    %GOTO Finish;
  %END;

%*~~~~~~~~~~~~~~~~~;
%* Parameter checks;
%*_________________;

  %LET OrigLen = %LENGTH(&Original);

  %IF (&Original EQ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Original value illegally is empty;
    %PUT .                  macro &MacName will abort;
    %GOTO Finish;
  %END;
  %ELSE %IF (%QSUBSTR(&Original,1,1) EQ .) %THEN
  %DO;                                    
    %IF (&OrigLen GT 2) %THEN
    %DO;
      %PUT *** &MacName *** &ERR: Original=&Original contains too many
 excessive character(s) %SUBSTR(&Original,3),;
      %PUT .                       macro &MacName will abort;
      %GOTO Finish;
    %END;
    %ELSE %IF (&OrigLen EQ 2) %THEN
    %DO;
      %IF (%INDEX(_ABCDEFGHIJKLMNOPQRSTUVWXYZ,%UPCASE(%SUBSTR(&Original,2))) EQ
        0) %THEN
      %DO;
        %PUT *** &MacName *** &ERR: Original=&Original contains illegal 2nd
 character %SUBSTR(&Original,2),;
        %PUT .                       macro &MacName will abort;
        %GOTO Finish;
      %END;
    %END;
  %END;
  %ELSE %IF (%QSUBSTR(&Original,1,1) EQ %STR(-)) %THEN
  %DO;
    %IF (%LENGTH(%SYSFUNC(COMPRESS(%SUBSTR(&Original,2), 0123456789))) GT 0) %THEN
    %DO;
      %PUT *** &MacName *** &ERR: Original=&Original contains illegal character(s)
 %SYSFUNC(COMPRESS(%SUBSTR(&Original,2), 0123456789)),;
      %PUT .                       macro &MacName will abort;
      %GOTO Finish;
    %END;
  %END;
  %ELSE %IF (%LENGTH(%SYSFUNC(COMPRESS(&Original, 0123456789))) GT 0) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Original=&Original contains illegal character(s)
 %SYSFUNC(COMPRESS(&Original, 0123456789)),;
    %PUT .                       macro &MacName will abort;
    %GOTO Finish;
  %END;

  %IF (&FixedLen EQ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: FixedLen value illegally is empty;
    %PUT .                  macro &MacName will abort;
    %GOTO Finish;
  %END;
  %ELSE %IF (%LENGTH(%SYSFUNC(COMPRESS(&FixedLen, 0123456789))) GT 0) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: FixedLen=&FixedLen contains illegal character(s)
 %SYSFUNC(COMPRESS(&FixedLen, 0123456789)),;
    %PUT .                       macro &MacName will abort;
    %GOTO Finish;
  %END;

  %IF (&OrigLen GT &FixedLen AND &FixedLen NE 0 AND
       %QSUBSTR(&Original,1,1) NE .) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Length of original value (&OrigLen) is larger;
    %PUT .                  than FixedLen (&FixedLen), macro &MacName will abort;
    %GOTO Finish;
  %END;


%*~~~~~~;
%* Core ;
%*______;

  %IF (&Original EQ .) %THEN %DO;
    %IF (&FixedLen LE 1) %THEN _;            %* generating _;
    %ELSE %SYSFUNC(REPEAT(_, &FixedLen-2))0; %* generating _0;
  %END;

  %ELSE %IF (%QSUBSTR(&Original,1,1) EQ .) %THEN
  %DO;
    %IF (&FixedLen LE 1) %THEN %SUBSTR(&Original,2); %* generating _ or letter;
    %ELSE %IF (&OrigLen LE &FixedLen) %THEN /* generating __ or _letter */
      %SYSFUNC(REPEAT(_,&FixedLen-&OrigLen))%SUBSTR(&Original,2);
  %END; %* [No discrimination between original . and ._];

  %ELSE %IF (&Original LT 0) %THEN
  %DO;  %* Original negative;
    %IF (&OrigLen LT %MAX(&FixedLen,1)) %THEN /* FixedLen = 1 / 0 */
      _%SYSFUNC(REPEAT(0,%MAX(&FixedLen,1)-1-&OrigLen))%EVAL(-&Original);
    %ELSE _%EVAL(-&Original);
  %END;

  %ELSE %IF (&OrigLen LT %MAX(&FixedLen,1)) %THEN /* Original non-negative */
    %SYSFUNC(REPEAT(0,%MAX(&FixedLen,1)-1-&OrigLen))&Original;

  %ELSE &Original;

%Finish:

%*~~~~~~~~~~~~;
%MEND PreZeros;
%*____________;

%*~~~~~~~~~~~~~~~~~~;
%* Auxiliary macros ;
%*------------------;
%* Macro MAX ;
%*___________;

%MACRO MAX / PARMBUFF;
  %SYSFUNC(MAX&SYSPBUFF) /* no semicolon! */
%MEND MAX;
